/**
 * Several tools exists for testing / updating / providing information over
 * the CAN bus. All these tools perform similiar initialization, timer, canbus
 * etc specific to the platform. The intention of a "task" is to seperate all
 * platform specific code from the rest. Especially for test code snippets, it
 * is quite helpfull if they can run instantly on all platforms.
 */

#include <velib/base/base.h>

#include <errno.h>
#include <signal.h>
#include <sys/time.h>
#include <stdlib.h>
#include <stdio.h>
#include <string.h>
#include <unistd.h>

#include <velib/platform/task.h>
#include <velib/platform/plt.h>
#include <velib/types/ve_stamp.h>
#include <velib/utils/ve_logger.h>
#include <velib/utils/ve_todo.h>

#include "platform_init.h"

static sig_atomic_t isTick;
static struct sigaction sa;
static struct itimerval timer;
static veBool veTodoFlag;

void sighand(int sig)
{
	switch (sig)
	{
	case SIGINT:
		//quit = 1;
		break;
	case SIGALRM:
		isTick = veTrue;
		break;
	}
}

int main(int argc, char *argv[])
{
	consoleInit(argc, argv);

	/* Install timer_handler as the signal handler for SIGVTALRM.  */
	memset(&sa, 0, sizeof(sa));
	sa.sa_handler = &sighand;
	sigaction(SIGALRM, &sa, NULL);

	/* Configure the timer to expire after 0 msec...  */
	timer.it_value.tv_sec = 0;
	timer.it_value.tv_usec = 1;

	/* ... and every 50 msec after that.  */
	timer.it_interval.tv_sec = 0;
	timer.it_interval.tv_usec = 50000;

#if !CFG_MAIN_NO_SIGNALS
	setitimer(ITIMER_REAL, &timer, NULL);
#endif

	for (;;)
	{
		/*
		 * Gdb stops on signals, at least in qt creator, so delay with
		 * sleep instead of using the wall time.
		 */
#if CFG_MAIN_NO_SIGNALS
		usleep(50000);
		isTick = veTrue;
#else
		pltIdleMode();
#endif

		if (isTick) {
			isTick = veFalse;
			taskTick();
		}

		do {
			veTodoFlag = veFalse;
			taskUpdate();
		} while(veTodoFlag);
	}

	return 0;
}

void veTodo(void)
{
	veTodoFlag = veTrue;
}

veBool veIsTodo(void)
{
	return veTodoFlag;
}
